#include <math.h>
#include <string>
#include <sstream>
#include "Plots_TWOFNR.h"
#include "Plots_KnownPeakFitter.h"
#include "DefineColours.h"
using namespace std;

//=====================================================================//
//=====================================================================//

vector<string> files;
TChain* tree_cd=NULL;
TChain* tree_au=NULL;
TChain* tree_auA=NULL;
TChain* tree_auB=NULL;
TChain* tree_both=NULL;
TChain* tree_raw=NULL;
//TChain* tree=NULL;
TChain* tree_PSp00=NULL;
TChain* tree_PSp96=NULL;

NPL::Reaction O19dp("19O(d,p)20O@151"); //TEMPORARY BEAM ENERGY!

auto colourR = new TColor((Int_t)TColor::GetFreeColorIndex(), 220./256.,   5./256.,  12./256.);
auto colourB = new TColor((Int_t)TColor::GetFreeColorIndex(),  25./256., 101./256., 176./256.);
auto colourG = new TColor((Int_t)TColor::GetFreeColorIndex(),  78./256., 178./256., 101./256.);
auto colourP = new TColor((Int_t)TColor::GetFreeColorIndex(), 136./256.,  46./256., 114./256.);

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void PlotKinematic(NPL::Reaction r, double Ex, Color_t c, int w, int s)
{
  r.SetExcitation4(Ex);
  TGraph* g = r.GetKinematicLine3();
  g->SetLineColor(c);
  g->SetLineWidth(w);
  g->SetLineStyle(s);
  g->Draw("c same");
}

//////////////////////////////////////////////////////////////////////////////////////
TChain* Chain(string TreeName, vector<string>& file, bool EventList)
{
  TChain* chain = new TChain(TreeName.c_str());
  unsigned int size = file.size();
  for(unsigned int i=0; i<size; i++){
    cout << "Adding to chain: " << file[i] << endl;
    chain->Add(file[i].c_str());
  }
  return chain;
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_Both_PaxmanAnalysis()
{
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_CD.root");
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-1.root"); 
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-2.root"); 
  tree_both = Chain("PhysicsTree",files,true);

  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_CD.root");
  tree_cd = Chain("PhysicsTree",files,true);

  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-1.root");
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-2.root");
  tree_au = Chain("PhysicsTree",files,true);
  
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-1.root");
  tree_auA = Chain("PhysicsTree",files,true);
  
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/19Odp_2024-10-31_Au-2.root");
  tree_auB = Chain("PhysicsTree",files,true);
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_Raw()
{
  files.clear();


  files.push_back("~/Experiment/e775s/RawData/run_0115/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0116/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0117/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0123/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0124/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0125/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0128/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0129/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0130/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0131/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0132/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0133/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0134/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0135/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0137/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0138/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0139/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0140/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0142/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0150/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0151/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0152/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0153/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0163/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0164/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0165/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0166/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0167/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0168/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0169/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0171/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0172/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0173/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0174/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0178/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0179/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0180/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0181/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0184/Out/Analysis/Tree_0000.root");
  files.push_back("~/Experiment/e775s/RawData/run_0185/Out/Analysis/Tree_0000.root");
  tree_raw = Chain("TreeMaster",files,true);
}

//////////////////////////////////////////////////////////////////////////////////////
void LoadChain_PhaseSpace()
{
  files.clear();
  files.push_back("../../../Outputs/Analysis/PSp_PhaseSpace_0000.root");
  tree_PSp00 = Chain("PhysicsTree",files,true);


  cout << " !!!! CHECK USE OF EVENT WEIGHT!! THINK I NEED TO FIX THIS!!! " << endl;
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_AddBack(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_AddBack","cExEg_AddBack",500,500);
  tree->Draw("Ex:AddBack_EDC>>hExEg(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExEg = (TH2F*) gDirectory->Get("hExEg");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEg->SetName("hExEg_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEg->SetName("hExEg_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEg->SetName("hExEg_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_Tracked(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_Tracked","cExEg_Tracked",500,500);
  tree->Draw("Ex:trackEDC>>hExEt(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExEt= (TH2F*) gDirectory->Get("hExEt");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEt->SetName("hExEt_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEt->SetName("hExEt_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEt->SetName("hExEt_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_TrackedMugast(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_trackM","cExEg_trackM",500,500);
  tree->Draw("Ex:trackEDC>>hExEtM(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExEtM= (TH2F*) gDirectory->Get("hExEtM");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEtM->SetName("hExEtM_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEtM->SetName("hExEtM_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEtM->SetName("hExEtM_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_TrackedVamos(TChain* tree)
{
  TCanvas *c = new TCanvas("cExEg_trackV","cExEg_trackV",500,500);
  tree->Draw("Ex:goldEDC>>hExEtV(8000,0,8,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExEtV= (TH2F*) gDirectory->Get("hExEtV");
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExEtV->SetName("hExEtV_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExEtV->SetName("hExEtV_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExEtV->SetName("hExEtV_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_Ex(TChain* tree)
{
  TCanvas *c = new TCanvas("cEx","cEx",500,500);
  tree->Draw("Ex>>hExGood(2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH1F* hExGood = (TH1F*) gDirectory->Get("hExGood");
  tree->Draw("Ex>>hExBad(2000,-5,15)","abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH1F* hExBad = (TH1F*) gDirectory->Get("hExBad");
  TH1F* hExClean = (TH1F*) hExGood->Clone();
  hExClean->SetTitle("hExClean");
  hExClean->SetName("hExClean");
  hExClean->Add(hExBad,-1);
  hExClean->Draw();
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExClean->SetName("hExClean_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExClean->SetName("hExClean_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExClean->SetName("hExClean_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExThetaLab(TChain* tree)
{ 
  TCanvas *c = new TCanvas("cExThetaLab","cExThetaLab",500,500);
  tree->Draw("Ex:ThetaLab>>hExTLGood(80,100,180,2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExTLGood = (TH2F*) gDirectory->Get("hExTLGood");
  tree->Draw("Ex:ThetaLab>>hExTLBad(80,100,180,2000,-5,15)","abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hExTLBad = (TH2F*) gDirectory->Get("hExTLBad");
  TH2F* hExTLClean = (TH2F*) hExTLGood->Clone();
  hExTLClean->SetTitle("hExTLClean");
  hExTLClean->SetName("hExTLClean");
  hExTLClean->Add(hExTLBad,-1);
  hExTLClean->Draw();
       if( tree->GetEntries() == tree_both->GetEntries() ) { hExTLClean->SetName("hExTLClean_both"); }
  else if( tree->GetEntries() == tree_cd->GetEntries() )   { hExTLClean->SetName("hExTLClean_cd"); }
  else if( tree->GetEntries() == tree_au->GetEntries() )   { hExTLClean->SetName("hExTLClean_au"); }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_All(TChain* tree)
{
//DrawPlotsToLoad_ExEg_AddBack(tree);
//DrawPlotsToLoad_ExEg_Tracked(tree);
  DrawPlotsToLoad_ExEg_TrackedMugast(tree);
  DrawPlotsToLoad_ExEg_TrackedVamos(tree);
  DrawPlotsToLoad_Ex(tree);
  DrawPlotsToLoad_ExThetaLab(tree);
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawPlotsToLoad_ExEg_AddBack(){ DrawPlotsToLoad_ExEg_AddBack(tree_both); }
void DrawPlotsToLoad_ExEg_Tracked(){ DrawPlotsToLoad_ExEg_Tracked(tree_both); }
void DrawPlotsToLoad_ExEg_TrackedMugast(){ DrawPlotsToLoad_ExEg_TrackedMugast(tree_both); }
void DrawPlotsToLoad_ExEg_TrackedVamos(){ DrawPlotsToLoad_ExEg_TrackedVamos(tree_both); }
void DrawPlotsToLoad_Ex(){ DrawPlotsToLoad_Ex(tree_both); }
void DrawPlotsToLoad_ExThetaLab(){ DrawPlotsToLoad_ExThetaLab(tree_both); }
void DrawPlotsToLoad_All(){ DrawPlotsToLoad_All(tree_both); }

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
TH1F* Project(TH2F* h2D, const char * XY, Color_t col)
{
  TH1F* h;
  if(string(XY) == "X"){
    string name = "projX_" + to_string(col);
    h2D->ProjectionX(name.c_str(),0,-1,"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else if (string(XY) == "Y"){
    string name = "projY_" + to_string(col);
    h2D->ProjectionY(name.c_str(),0,-1,"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else {
    cout << "!!!! ERROR !!!! XY projection selector is invalid" << endl;
    return 0;
  }
  return (TH1F*) h;
}

//////////////////////////////////////////////////////////////////////////////////////
TH1F* Project(TH2F* h2D, const char * XY, double mean, double width, Color_t col)
{
  TH1F* h;
  if(string(XY) == "X"){
    //X projection is GPSG
    string name = "projX_" + to_string(col);
    h2D->ProjectionX(name.c_str(),h2D->GetYaxis()->FindBin(mean-(0.5*width)),
                                  h2D->GetYaxis()->FindBin(mean+(0.5*width)),"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else if (string(XY) == "Y"){
    //Y projection is GGSP
    string name = "projY_" + to_string(col);
    h2D->ProjectionY(name.c_str(),h2D->GetXaxis()->FindBin(mean-(0.5*width)),
                                  h2D->GetXaxis()->FindBin(mean+(0.5*width)),"");
    h = (TH1F*) gDirectory->Get(name.c_str());
    h->SetLineColor(col);
  } else {
    cout << "!!!! ERROR !!!! XY projection selector is invalid" << endl;
    return 0;
  }
  return (TH1F*) h;
}

//////////////////////////////////////////////////////////////////////////////////////
void Prj_DrawLines(double mean, double width, Color_t col)
{
  TLine* line = new TLine();
  line->SetLineColor(col);
  line->SetLineStyle(kDashed);
  line->SetLineWidth(2);
  line->DrawLine(mean-(0.5*width), 0, mean-(0.5*width), 2000);
  line->DrawLine(mean+(0.5*width), 0, mean+(0.5*width), 2000);
}

//////////////////////////////////////////////////////////////////////////////////////
string MidCDOrPostAu(string input)
{
       if(input == "midCD"){  return "hExEtM"; }
  else if(input == "postAu"){ return "hExEtV"; }
  else { cout << "ERROR! not 'midCD' or 'postAu' !!!" << endl; return 0;}
}

//////////////////////////////////////////////////////////////////////////////////////
void CheckExEgExists(string histname, string tree, string midcdOrPostau)
{
  if(gDirectory->Get(histname.c_str()) == 0){
    if(midcdOrPostau == "midCD") { 
      if(tree == "cd")   { DrawPlotsToLoad_ExEg_TrackedMugast(tree_cd);   }
      if(tree == "au")   { DrawPlotsToLoad_ExEg_TrackedMugast(tree_au);   }
      if(tree == "both") { DrawPlotsToLoad_ExEg_TrackedMugast(tree_both); }
    } else { 
      if(tree == "cd")   { DrawPlotsToLoad_ExEg_TrackedVamos(tree_cd);    }
      if(tree == "au")   { DrawPlotsToLoad_ExEg_TrackedVamos(tree_au);    }
      if(tree == "both") { DrawPlotsToLoad_ExEg_TrackedVamos(tree_both);  }
    }
  }
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Eg1, double width1)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  projP1->Draw();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  projP1->Draw();
  projP2->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3,
                                                     double Eg4, double width4)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);
  Prj_DrawLines(Eg4, width4, kViolet);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  TH1F* projP4 = Project(hExEg, "Y", Eg4, width4, kViolet);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
  projP4->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Eg1, double width1,
                                                     double Eg2, double width2,
                                                     double Eg3, double width3,
                                                     double Eg4, double width4,
                                                     double Eg5, double width5)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGGSP = new TCanvas("cGGSP","cGGSP",1000,750);
  cGGSP->Divide(1,2);
  cGGSP->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gates
  Prj_DrawLines(Eg1, width1, kRed);
  Prj_DrawLines(Eg2, width2, kBlue);
  Prj_DrawLines(Eg3, width3, kGreen);
  Prj_DrawLines(Eg4, width4, kViolet);
  Prj_DrawLines(Eg5, width5, kOrange);

  //Projected axis
  cGGSP->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", Eg1, width1, kRed);
  TH1F* projP2 = Project(hExEg, "Y", Eg2, width2, kBlue);
  TH1F* projP3 = Project(hExEg, "Y", Eg3, width3, kGreen);
  TH1F* projP4 = Project(hExEg, "Y", Eg4, width4, kViolet);
  TH1F* projP5 = Project(hExEg, "Y", Eg5, width5, kOrange);
  projP1->Draw();
  projP2->Draw("same");
  projP3->Draw("same");
  projP4->Draw("same");
  projP5->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Ex1, double width1)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  projG1->Draw();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  projG1->Draw();
  projG2->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3,
					             double Ex4, double width4)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);
  Prj_DrawLines(Ex4, width4, kViolet);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  TH1F* projG4 = Project(hExEg, "X", Ex4, width4, kViolet);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
  projG4->Draw("same");
}

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_SeeGamma(string treesuffix, string midcdOrPostau, double bing, double binp, 
                                                     double Ex1, double width1,
					             double Ex2, double width2,
					             double Ex3, double width3,
					             double Ex4, double width4,
					             double Ex5, double width5)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG = new TCanvas("cGPSG","cGPSG",1000,750);
  cGPSG->Divide(1,2);
  cGPSG->cd(1);

  //Axis being projected
  TH1F* projP = Project(hExEg, "Y", kBlack);
  projP->Draw();
 
  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);
  Prj_DrawLines(Ex2, width2, kBlue);
  Prj_DrawLines(Ex3, width3, kGreen);
  Prj_DrawLines(Ex4, width4, kViolet);
  Prj_DrawLines(Ex5, width5, kOrange);

  //Projected axis
  cGPSG->cd(2);
  TH1F* projG1 = Project(hExEg, "X", Ex1, width1, kRed);
  TH1F* projG2 = Project(hExEg, "X", Ex2, width2, kBlue);
  TH1F* projG3 = Project(hExEg, "X", Ex3, width3, kGreen);
  TH1F* projG4 = Project(hExEg, "X", Ex4, width4, kViolet);
  TH1F* projG5 = Project(hExEg, "X", Ex5, width5, kOrange);
  projG1->Draw();
  projG2->Draw("same");
  projG3->Draw("same");
  projG4->Draw("same");
  projG5->Draw("same");
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void NoParticleGate_CompareTargets(double bing)
{
  string histplot = "(" + to_string(8./bing) + ",0,8)";
  string drawAuAu = "goldEDC>>hAuAu"  +histplot;
  string drawAuCD = "trackEDC>>hAuCD" +histplot;
  string drawCDCD = "trackEDC>>hCDCD" +histplot;

  tree_au->Draw(drawAuAu.c_str(),"abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
    TH1F* hAuAu = (TH1F*) gDirectory->Get("hAuAu");
    hAuAu->SetLineColor(kOrange+2);
  tree_au->Draw(drawAuCD.c_str(),"abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
    TH1F* hAuCD = (TH1F*) gDirectory->Get("hAuCD");
    hAuCD->SetLineColor(kCyan+2);
  tree_cd->Draw(drawCDCD.c_str(),"abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
    TH1F* hCDCD = (TH1F*) gDirectory->Get("hCDCD");
    hCDCD->SetLineColor(kBlack);

  //Set up canvas
  TCanvas *cNPCT = new TCanvas("cNPCT","cNPCT",1000,750);

  hAuAu->Draw();
  hAuCD->Draw("same");
  hCDCD->Draw("same");
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateParticle_CompareTargets(double bing, double binp, 
                                 double Ex1, double width1)
{
  string histname_cd      = MidCDOrPostAu("midCD");   histname_cd += "_cd";
  CheckExEgExists(histname_cd,      "cd", "midCD");
  
  string histname_au_mid  = MidCDOrPostAu("midCD");   histname_au_mid += "_au";
  CheckExEgExists(histname_au_mid,  "au", "midCD");
  
  string histname_au_post = MidCDOrPostAu("postAu");  histname_au_post += "_au";
  CheckExEgExists(histname_au_post, "au", "postAu");

  //Load in & rebin CD
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname_cd.c_str());
  TH2F* hExEg_cd = (TH2F*) hCopyMe->Clone();
  hExEg_cd->Rebin2D(bing/hExEg_cd->GetXaxis()->GetBinWidth(3),
                    binp/hExEg_cd->GetYaxis()->GetBinWidth(3));

  //Load in & rebin Au mid
        hCopyMe = (TH2F*) gDirectory->Get(histname_au_mid.c_str());
  TH2F* hExEg_au_mid = (TH2F*) hCopyMe->Clone();
  hExEg_au_mid->Rebin2D(bing/hExEg_au_mid->GetXaxis()->GetBinWidth(3),
                        binp/hExEg_au_mid->GetYaxis()->GetBinWidth(3));
  
  //Load in & rebin Au post
        hCopyMe = (TH2F*) gDirectory->Get(histname_au_post.c_str());
  TH2F* hExEg_au_post = (TH2F*) hCopyMe->Clone();
  hExEg_au_post->Rebin2D(bing/hExEg_au_post->GetXaxis()->GetBinWidth(3),
                         binp/hExEg_au_post->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPCT = new TCanvas("cGPCT","cGPCT",1000,750);
  cGPCT->Divide(1,2);
  cGPCT->cd(1);

  //Axis being projected
  TH1F* projP_au = Project(hExEg_au_mid, "Y", kOrange+2);
  projP_au->Draw();
  TH1F* projP_cd = Project(hExEg_cd    , "Y", kBlack);
  projP_cd->SetFillColor(kBlack); projP_cd->SetFillStyle(3001); 
  projP_cd->Draw("same");

  //Projection gates
  Prj_DrawLines(Ex1, width1, kRed);

  //Projected axis
  cGPCT->cd(2);
  TH1F* projG1_au_mid  = Project(hExEg_au_mid , "X", Ex1, width1, kCyan+2);
  TH1F* projG1_au_post = Project(hExEg_au_post, "X", Ex1, width1, kOrange+2);
  TH1F* projG1_cd      = Project(hExEg_cd     , "X", Ex1, width1, kBlack);
    projG1_cd->SetFillColor(kBlack); projG1_cd->SetFillStyle(3001);

  projG1_au_mid ->Draw();
  projG1_au_post->Draw("same");
  projG1_cd     ->Draw("same");
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void GateGamma_SeeParticle_HighEnergyStaggered(string treesuffix, string midcdOrPostau, 
                                               double bing, double binp)
{
  string histname = MidCDOrPostAu(midcdOrPostau);
  histname = histname + "_" + treesuffix;
  CheckExEgExists(histname, treesuffix, midcdOrPostau);

  gStyle->SetPalette(kCool);

  double width = 8.0;
//vector<double> energies = {3.4,3.6,3.8,4.0,4.8}; // Good one!
  vector<double> energies = {3.45,3.60,4.1,5.6,6.1,6.6,7.7};
  auto cols = TColor::GetPalette();

  //Load in & rebin
  TH2F* hCopyMe = (TH2F*) gDirectory->Get(histname.c_str());
  TH2F* hExEg = (TH2F*) hCopyMe->Clone();
  hExEg->Rebin2D(bing/hExEg->GetXaxis()->GetBinWidth(3),
                 binp/hExEg->GetYaxis()->GetBinWidth(3));

  //Set up canvas
  TCanvas *cGPSG_High = new TCanvas("cGPSG_High","cGPSG_High",1000,750);
  cGPSG_High->Divide(1,2);
  cGPSG_High->cd(1);

  //Axis being projected
  TH1F* projG = Project(hExEg, "X", kBlack);
  projG->Draw();
 
  //Projection gatesi
  for(int i=0; i<energies.size(); i++){
    Prj_DrawLines(energies.at(i)+width/2., width, cols.At(i));
  }

  //Projected axis
  cGPSG_High->cd(2);
  TH1F* projP1 = Project(hExEg, "Y", energies.at(0)+width/2., width, cols.At(0));
  projP1->Draw("plc");
  for(int i=1; i<energies.size(); i++){
    TH1F* projP = Project(hExEg, "Y", energies.at(i)+width/2., width, cols.At(i));
    projP->Draw("plc same");
  } 

}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void DrawExThetaLab_SubPSp(TChain* tree)
{
  LoadChain_PhaseSpace();
  TCanvas* c = new TCanvas("c","c",500,500);

  // ExTL Good
  tree->Draw("Ex:ThetaLab>>hGood(26,104,156,150,-2,13)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  // ExTL Bad
  tree->Draw("Ex:ThetaLab>>hBad(26,104,156,150,-2,13)","abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2","colz");
  TH2F* hBad = (TH2F*) gDirectory->Get("hBad");
  
  // ExTL Clean
  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);

  // ExTL PSp
  tree_PSp00->Draw("Ex:ThetaLab>>hPSp(26,104,156,150,-2,13)","EventWeight*(Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2)","colz");
  TH2F* hPSp = (TH2F*) gDirectory->Get("hPSp");

  double scale = 1.0;
  int minX = hGood->GetXaxis()->FindBin(106);//110);
  int maxX = hGood->GetXaxis()->FindBin(120);//135);//130);//145);
  int minY = hGood->GetYaxis()->FindBin(7.0);
  int maxY = hGood->GetYaxis()->FindBin(10.5);
  int step = (maxX-minX)/3;
   
  cout << minX << " " << maxX << "  ||  " << minY << " " << maxY << endl;

  for(int x = minX; x < maxX; x++){
//  cout << " ---- X = " << x << " ---- " << endl;
    for(int y = minY; y < maxY; y++){
//    cout << " ---- Y = " << y << " ---- " << endl;

      //if(hClean->GetYaxis()->GetBinCenter(y) < (-0.0833 * hClean->GetXaxis()->GetBinCenter(x) ) + 17.75){
      if(hClean->GetYaxis()->GetBinCenter(y) < (-0.0833 * hClean->GetXaxis()->GetBinCenter(x) ) + 18.75){
//        cout << x << " " << y << " || " << hPSp->GetBinContent(x,y) << " vs " << hClean->GetBinContent(x,y) << endl;

        if(hClean->GetBinContent(x,y)>2){
          while(hPSp->GetBinContent(x,y) * scale > hClean->GetBinContent(x,y)){
            scale *= 0.999;
            cout << hPSp->GetXaxis()->GetBinCenter(x) << ", " << hPSp->GetYaxis()->GetBinCenter(y) 
	         << " | " << hPSp->GetBinContent(x,y) * scale << " > " << hClean->GetBinContent(x,y) 
		 << " | " << scale << endl;
          }
        }

      }
    }
  }

  cout << " |||| scale = " << scale << " |||| " << endl;
  delete c;

  TCanvas* cClean = new TCanvas("cClean","cClean",500,500);
  hClean->Draw("colz");

  TCanvas* cPSp = new TCanvas("cPSp","cPSp",500,500);
  hPSp->Draw("colz");

  TCanvas* cFinal = new TCanvas("cFinal","cFinal",500,500);
  TH2F* hFinal = (TH2F*) hClean->Clone();
  hFinal->SetTitle("hFinal");
  hFinal->SetName("hFinal");
  hFinal->Add(hPSp,-scale);
  hFinal->Draw("colz");

  TCanvas* cProj = new TCanvas("cProj","cProj",1000,500);
  TH1F* hLowC = (TH1F*) hClean->ProjectionY("LC", minX,      minX+step);
  TH1F* hMidC = (TH1F*) hClean->ProjectionY("MC", minX+step, maxX-step);
  TH1F* hHghC = (TH1F*) hClean->ProjectionY("HC", maxX-step, maxX     );
 
  hLowC->SetLineColor(kBlack);
  hMidC->SetLineColor(kBlack);
  hHghC->SetLineColor(kBlack);

  TH1F* hLowP = (TH1F*) hPSp->ProjectionY("LP", minX,      minX+step);
  TH1F* hMidP = (TH1F*) hPSp->ProjectionY("MP", minX+step, maxX-step);
  TH1F* hHghP = (TH1F*) hPSp->ProjectionY("HP", maxX-step, maxX     );

  hLowP->SetLineColor(kRed);  hLowP->Scale(scale);
  hMidP->SetLineColor(kRed);  hMidP->Scale(scale);
  hHghP->SetLineColor(kRed);  hHghP->Scale(scale);

  TH1F* hLowF = (TH1F*) hFinal->ProjectionY("LF", minX,      minX+step);
  TH1F* hMidF = (TH1F*) hFinal->ProjectionY("MF", minX+step, maxX-step);
  TH1F* hHghF = (TH1F*) hFinal->ProjectionY("HF", maxX-step, maxX     );

  hLowF->SetLineColor(kBlue);
  hMidF->SetLineColor(kBlue);
  hHghF->SetLineColor(kBlue);

  cProj->Divide(3);
  cProj->cd(1); hLowC->Draw(); hLowP->Draw("same"); hLowF->Draw("same");
  cProj->cd(2); hMidC->Draw(); hMidP->Draw("same"); hMidF->Draw("same");
  cProj->cd(3); hHghC->Draw(); hHghP->Draw("same"); hHghF->Draw("same");

}

//////////////////////////////////////////////////////////////////////////////////////
void DrawEx_SubPSp(TChain* tree)
{
  LoadChain_PhaseSpace();
  TCanvas* c = new TCanvas("c","c",1000,1000);

  // ExTL Good
  tree->Draw("Ex>>hGood(2000,-5,15)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH1F* hGood = (TH1F*) gDirectory->Get("hGood");
  
  // ExTL Bad
  tree->Draw("Ex>>hBad(2000,-5,15)","abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH1F* hBad = (TH1F*) gDirectory->Get("hBad");
  
  // ExTL Clean
  TH1F* hClean = (TH1F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);

  // ExTL PSp
  tree_PSp00->Draw("Ex>>hPSp(2000,-5,15)","EventWeight*(Mugast.TelescopeNumber<9)","colz");
  TH1F* hPSp = (TH1F*) gDirectory->Get("hPSp");

//double scale = 0.134256;
//double scale = 0.00113356;
  double scale = 0.00216125;
  cout << " || SUBTRACTING PHASE SPACE WITH SCALE = " << scale << endl;

  TH1F* hFinal = (TH1F*) hClean->Clone();
  hFinal->SetTitle("hFinal");
  hFinal->SetName("hFinal");
  hFinal->Add(hPSp,-scale);

  hClean->SetLineColor(kBlack);                       hClean->Draw();
  hPSp  ->SetLineColor(kRed);    hPSp->Scale(scale);  hPSp  ->Draw("same");
  hFinal->SetLineColor(kBlue);                        hFinal->Draw("same");

}

//////////////////////////////////////////////////////////////////////////////////////
void DrawEx_SelectMWN(TChain* tree, int lowMWN, int highMWN)
{
  TCanvas* c = new TCanvas("c","c",1000,1000);

  // ExTL Good
  string goodstring = "abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9 && MW_N>=" + to_string(lowMWN) 
                                                                         + " && MW_N<=" + to_string(highMWN);
  tree->Draw("Ex>>hGood(2000,-5,15)",goodstring.c_str(),"colz");
  TH1F* hGood = (TH1F*) gDirectory->Get("hGood");
  
  // ExTL Bad
  string badstring = "abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9 && MW_N>=" + to_string(lowMWN)
                                                                        + " && MW_N<=" + to_string(highMWN);
  tree->Draw("Ex>>hBad(2000,-5,15)",badstring.c_str(),"colz");
  TH1F* hBad = (TH1F*) gDirectory->Get("hBad");
  
  // ExTL Clean
  TH1F* hClean = (TH1F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->SetLineColor(kBlack);                       hClean->Draw();

}

//////////////////////////////////////////////////////////////////////////////////////
void AddKnownStateLines(TH1F* hist, double ymax)
{
  hist->Draw();

  TLine* Sn = new TLine(7.608, 0.0,7.608, ymax);
  Sn->SetLineColor(kOrange);
  Sn->SetLineWidth(4);
  Sn->SetLineStyle(kDashed);
  Sn->Draw();

  for(int i=0; i<numPeaks; i++){
    TLine* l = new TLine(means.at(i), 0.0, means.at(i), ymax);
    l->SetLineColor(kBlack);
    l->SetLineWidth(2);
    l->SetLineStyle(kDotted);
    l->Draw();
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void AddKnownGammas(TH1F* hist, double ymax)
{
  vector<double> gammas = {0.096,1.154,1.675,1.897,2.397,3.551,3.954,4.071,5.018,5.305};

  for(int i=0; i<gammas.size(); i++){
    TLine* l = new TLine(gammas.at(i), 0.0, gammas.at(i), ymax);
    l->SetLineColor(kBlack);
    l->SetLineWidth(2);
    l->SetLineStyle(kDotted);
    l->Draw();
  }
}

//////////////////////////////////////////////////////////////////////////////////////
void DrawGammas_AddBackVersusTracked()
{
  TCanvas* cGammaCompare = new TCanvas("cGammaCompare","cGammaCompare",1600,800);

  tree_both->Draw("trackEDC>>hTracked(5000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9",
		  "");
  TH1F* hTracked = (TH1F*) gDirectory->Get("hTracked");
  hTracked->SetLineColor(kRed);
  hTracked->SetLineWidth(2);

  
  tree_both->Draw("AddBack_EDC>>hAddBack(5000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9",
		  "");
  TH1F* hAddBack = (TH1F*) gDirectory->Get("hAddBack");
  hAddBack->SetLineColor(kBlue);
  hAddBack->SetLineWidth(2);

  hTracked->Draw(); hAddBack->Draw("same");

}

void DrawGammas_MidTargVersusPostAu()
{
  TCanvas* cGammaCompare = new TCanvas("cGammaCompare","cGammaCompare",1600,800);

  tree_both->Draw("trackEDC>>hTrackM(5000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9",
		  "");
  TH1F* hTrackM = (TH1F*) gDirectory->Get("hTrackM");
  hTrackM->SetLineColor(kRed);
  hTrackM->SetLineWidth(2);

  
  tree_both->Draw("goldEDC>>hTrackV(5000,-1,9)",
                  "Mugast.TelescopeNumber<9 && abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9",
		  "");
  TH1F* hTrackV = (TH1F*) gDirectory->Get("hTrackV");
  hTrackV->SetLineColor(kBlue);
  hTrackV->SetLineWidth(2);

  hTrackM->Draw(); hTrackV->Draw("same");

}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void Diagnostics_StripNumber(TTree* tree)
{
  TCanvas* c = new TCanvas("c","c",1000,1000);

  c->Divide(4,4);
  
  c->cd(1);
  tree->Draw("Mugast.DSSD_X>>hX1(130,0,130)","Mugast.TelescopeNumber==1","");
  c->cd(2);
  tree->Draw("Mugast.DSSD_Y>>hY1(130,0,130)","Mugast.TelescopeNumber==1","");
  
  c->cd(3);
  tree->Draw("Mugast.DSSD_X>>hX5(130,0,130)","Mugast.TelescopeNumber==5","");
  c->cd(4);
  tree->Draw("Mugast.DSSD_Y>>hY5(130,0,130)","Mugast.TelescopeNumber==5","");
  
  c->cd(5);
  tree->Draw("Mugast.DSSD_X>>hX2(130,0,130)","Mugast.TelescopeNumber==2","");
  c->cd(6);
  tree->Draw("Mugast.DSSD_Y>>hY2(130,0,130)","Mugast.TelescopeNumber==2","");

  c->cd(7);
  tree->Draw("Mugast.DSSD_X>>hX6(130,0,130)","Mugast.TelescopeNumber==6","");
  c->cd(8);
  tree->Draw("Mugast.DSSD_Y>>hY6(130,0,130)","Mugast.TelescopeNumber==6","");

  c->cd(9);
  tree->Draw("Mugast.DSSD_X>>hX3(130,0,130)","Mugast.TelescopeNumber==3","");
  c->cd(10);
  tree->Draw("Mugast.DSSD_Y>>hY3(130,0,130)","Mugast.TelescopeNumber==3","");

  c->cd(11);
  tree->Draw("Mugast.DSSD_X>>hX7(130,0,130)","Mugast.TelescopeNumber==7","");
  c->cd(12);
  tree->Draw("Mugast.DSSD_Y>>hY7(130,0,130)","Mugast.TelescopeNumber==7","");

  c->cd(13);
  tree->Draw("Mugast.DSSD_X>>hX4(130,0,130)","Mugast.TelescopeNumber==4","");
  c->cd(14);
  tree->Draw("Mugast.DSSD_Y>>hY4(130,0,130)","Mugast.TelescopeNumber==4","");



}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
		    
  tree_both->Draw(drawgood.c_str(),"abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str(), "abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_SubPSp(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  LoadChain_PhaseSpace();
  TChain* treePSp = tree_PSp00;

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		    
  tree_both->Draw(drawgood.c_str(),"abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str(), "abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9","colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  treePSp->Draw(drawpsp.c_str(),"EventWeight*(Mugast.TelescopeNumber<10)","colz");
  TH2F* hPSp  = (TH2F*) gDirectory->Get("hPSp");  hPSp->SetTitle("hPSp");  hPSp->SetName("hPSp");
//double scale = 0.134256; cout << GREEN << " USING SCALING FROM DrawEx_SubPSp, SCALE =  " << scale << RESET << endl;
//double scale = 0.00113356; cout << GREEN << " USING SCALING FROM DrawEx_SubPSp, SCALE =  " << scale << RESET << endl;
  double scale = 0.00216125; cout << GREEN << " USING SCALING FROM DrawEx_SubPSp, SCALE =  " << scale << RESET << endl;
  
  hClean->Add(hPSp, -scale);

  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GatePhaseSpace_MultiWrite(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  LoadChain_PhaseSpace();
  double finishTheta = startTheta + (numGates * gatesize);

  string draw = "Ex:ThetaLab>>h(" 
                + to_string(numGates) 
		+ "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		+ to_string(20.0/ExBinsize)
		+ ",-5,15)";
  
  tree_PSp00->Draw(draw.c_str(),"","colz");
  TH2F* h = (TH2F*) gDirectory->Get("h");
  h->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) h->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_PhaseSpace.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_ParallelCuts(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string baseParallelCuts = "(ELab < (0.05458*ThetaLab)-3.456) && (ELab > (0.05458*ThetaLab)-5.696) && Ex < 8.7 && Mugast.TelescopeNumber<9";

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		   
  string gategood = "abs(T_FPMW_HF-13300)<1500 &&" + baseParallelCuts;
  string gatebad  = "abs(T_FPMW_HF-10300)<1500 &&" + baseParallelCuts;

  tree_both->Draw(drawgood.c_str(), gategood.c_str(), "colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str() , gatebad.c_str() , "colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Parallel_Experiment.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//////////////////////////////////////////////////////////////////////////////////////
void GateThetaLab_MultiWrite_ParallelCuts_SubPSp(double startTheta, int numGates, double gatesize, double ExBinsize)
{
  double finishTheta = startTheta + (numGates * gatesize);

  string baseParallelCuts = "(ELab < (0.05458*ThetaLab)-3.456) && (ELab > (0.05458*ThetaLab)-5.696) && Ex < 8.7 && Mugast.TelescopeNumber<9";
  string PSpParallelCuts = "EventWeight*(" + baseParallelCuts + ")";

  LoadChain_PhaseSpace();
  TChain* treePSp = tree_PSp00;

  string drawbase = to_string(numGates) 
		    + "," + to_string(startTheta) + "," + to_string(finishTheta) + ","
		    + to_string(20.0/ExBinsize)
		    + ",-5,15)";
  string drawgood = "Ex:ThetaLab>>hGood(" + drawbase;
  string drawbad  = "Ex:ThetaLab>>hBad("  + drawbase;
  string drawpsp  = "Ex:ThetaLab>>hPSp("  + drawbase;
		   
  string gategood = "abs(T_FPMW_HF-13300)<1500 &&" + baseParallelCuts;
  string gatebad  = "abs(T_FPMW_HF-10300)<1500 &&" + baseParallelCuts;

  tree_both->Draw(drawgood.c_str(), gategood.c_str(), "colz");
  TH2F* hGood = (TH2F*) gDirectory->Get("hGood");
  
  tree_both->Draw(drawbad.c_str() , gatebad.c_str() , "colz");
  TH2F* hBad  = (TH2F*) gDirectory->Get("hBad");

  TH2F* hClean = (TH2F*) hGood->Clone();
  hClean->SetTitle("hClean");
  hClean->SetName("hClean");
  hClean->Add(hBad,-1);
  hClean->Draw("colz");
  
  treePSp->Draw(drawpsp.c_str(),PSpParallelCuts.c_str(),"colz");
  TH2F* hPSp  = (TH2F*) gDirectory->Get("hPSp");  hPSp->SetTitle("hPSp");  hPSp->SetName("hPSp");
  double scale = 1.0;
  for(int t=0; t<numGates; t++){
    for(int b=hClean->GetYaxis()->FindBin(7.7); b<hClean->GetYaxis()->FindBin(8.5);b++){
      while(hClean->GetBinContent(t,b)>2 && hClean->GetBinContent(t,b)<(hPSp->GetBinContent(t,b)*scale)){
        scale *= 0.999;
      }
    }
  }
  cout << "scale = " << scale << endl;

  hClean->Add(hPSp,-scale);

  string ytitle = "Counts / " + to_string(ExBinsize) + " MeV";
  TList* list = new TList();

  for (int i=0; i<numGates; i++){
    cout << "Writing gate " << i+1 << "/" << numGates << endl;
    double minTheta = startTheta + (i * gatesize);
    string title = "TLab"+to_string((int) minTheta)+"to"+to_string((int) (minTheta+gatesize));

    TH1F* Ex_ThetaLabGate = (TH1F*) hClean->ProjectionY(title.c_str(), i, i+1, "");
    Ex_ThetaLabGate->GetXaxis()->SetTitle("Ex [MeV]");
    Ex_ThetaLabGate->GetYaxis()->SetTitle(ytitle.c_str());
    Ex_ThetaLabGate->Sumw2();
    Ex_ThetaLabGate->SetTitle(title.c_str());
    list->Add(Ex_ThetaLabGate);
  }

  TFile* file = new TFile("./C2S_Outputs/GateThetaLabHist_Parallel_Experiment_SubPSp.root","RECREATE");
  list->Write("GateThetaLabHistograms",TObject::kSingleKey);
  file->ls();
}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void CompareExpSimStrips()
{

  TChain* tree_sim=NULL;
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/Sim_19Odp_1675.root");
  tree_sim = Chain("PhysicsTree",files,true);

  //*********************************************************************//

  TCanvas* cStrips = new TCanvas("cStrips","cStrips",2000,1000);
  cStrips->Divide(4,2);

  cStrips->cd(1);
    tree_both->Draw("Mugast.DSSD_X>>e1x(128,0,128)","Mugast.TelescopeNumber==1","");
    tree_sim->Draw( "Mugast.DSSD_X>>s1x(128,0,128)","Mugast.TelescopeNumber==1","same");
    TH1F* e1x = (TH1F*) gDirectory->Get(        "e1x");
    TH1F* s1x = (TH1F*) gDirectory->Get(        "s1x");
          s1x->GetYaxis()->SetRangeUser(0,8000);
          e1x->SetLineColor(kRed);               e1x->SetFillColor(kRed);
          s1x->SetLineColor(kOrange);            s1x->SetFillColor(kOrange);
          e1x->SetFillStyle(3002);               s1x->SetFillStyle(3001);
          s1x->DrawNormalized();                           e1x->DrawNormalized("same");

  cStrips->cd(2);
    tree_both->Draw("Mugast.DSSD_X>>e2x(128,0,128)","Mugast.TelescopeNumber==2","");
    tree_sim->Draw( "Mugast.DSSD_X>>s2x(128,0,128)","Mugast.TelescopeNumber==2","same");
    TH1F* e2x = (TH1F*) gDirectory->Get(        "e2x");
    TH1F* s2x = (TH1F*) gDirectory->Get(        "s2x");
          s2x->GetYaxis()->SetRangeUser(0,8000);
          e2x->SetLineColor(kRed);               e2x->SetFillColor(kRed);
          s2x->SetLineColor(kOrange);            s2x->SetFillColor(kOrange);
          e2x->SetFillStyle(3002);               s2x->SetFillStyle(3001);
          s2x->DrawNormalized();                           e2x->DrawNormalized("same");

 cStrips->cd(3);
    tree_both->Draw("Mugast.DSSD_X>>e3x(128,0,128)","Mugast.TelescopeNumber==3","");
    tree_sim->Draw( "Mugast.DSSD_X>>s3x(128,0,128)","Mugast.TelescopeNumber==3","same");

    TH1F* e3x = (TH1F*) gDirectory->Get(        "e3x");
    TH1F* s3x = (TH1F*) gDirectory->Get(        "s3x");
          s3x->GetYaxis()->SetRangeUser(0,8000);
          e3x->SetLineColor(kRed);               e3x->SetFillColor(kRed);
          s3x->SetLineColor(kOrange);            s3x->SetFillColor(kOrange);
          e3x->SetFillStyle(3002);               s3x->SetFillStyle(3001);
          s3x->DrawNormalized();                           e3x->DrawNormalized("same");

  cStrips->cd(4);
    tree_both->Draw("Mugast.DSSD_X>>e4x(128,0,128)","Mugast.TelescopeNumber==4","");
    tree_sim->Draw( "Mugast.DSSD_X>>s4x(128,0,128)","Mugast.TelescopeNumber==4","same");

    TH1F* e4x = (TH1F*) gDirectory->Get(        "e4x");
    TH1F* s4x = (TH1F*) gDirectory->Get(        "s4x");
          s4x->GetYaxis()->SetRangeUser(0,8000);
          e4x->SetLineColor(kRed);               e4x->SetFillColor(kRed);
          s4x->SetLineColor(kOrange);            s4x->SetFillColor(kOrange);
          e4x->SetFillStyle(3002);               s4x->SetFillStyle(3001);
          s4x->DrawNormalized();                           e4x->DrawNormalized("same");

  cStrips->cd(5);
    tree_both->Draw("Mugast.DSSD_X>>e5x(128,0,128)","Mugast.TelescopeNumber==5","");
    tree_sim->Draw( "Mugast.DSSD_X>>s5x(128,0,128)","Mugast.TelescopeNumber==5","same");

    TH1F* e5x = (TH1F*) gDirectory->Get(        "e5x");
    TH1F* s5x = (TH1F*) gDirectory->Get(        "s5x");
          s5x->GetYaxis()->SetRangeUser(0,8000);
          e5x->SetLineColor(kRed);               e5x->SetFillColor(kRed);
          s5x->SetLineColor(kOrange);            s5x->SetFillColor(kOrange);
          e5x->SetFillStyle(3002);               s5x->SetFillStyle(3001);
          s5x->DrawNormalized();                           e5x->DrawNormalized("same");

  cStrips->cd(6);
    tree_both->Draw("Mugast.DSSD_X>>e6x(128,0,128)","Mugast.TelescopeNumber==6","");
    tree_sim->Draw( "Mugast.DSSD_X>>s6x(128,0,128)","Mugast.TelescopeNumber==6","same");

    TH1F* e6x = (TH1F*) gDirectory->Get(        "e6x");
    TH1F* s6x = (TH1F*) gDirectory->Get(        "s6x");
          s6x->GetYaxis()->SetRangeUser(0,8000);
          e6x->SetLineColor(kRed);               e6x->SetFillColor(kRed);
          s6x->SetLineColor(kOrange);            s6x->SetFillColor(kOrange);
          e6x->SetFillStyle(3002);               s6x->SetFillStyle(3001);
          s6x->DrawNormalized();                           e6x->DrawNormalized("same");

  cStrips->cd(7);
    tree_both->Draw("Mugast.DSSD_X>>e7x(128,0,128)","Mugast.TelescopeNumber==7","");
    tree_sim->Draw( "Mugast.DSSD_X>>s7x(128,0,128)","Mugast.TelescopeNumber==7","same");

    TH1F* e7x = (TH1F*) gDirectory->Get(        "e7x");
    TH1F* s7x = (TH1F*) gDirectory->Get(        "s7x");
          s7x->GetYaxis()->SetRangeUser(0,8000);
          e7x->SetLineColor(kRed);               e7x->SetFillColor(kRed);
          s7x->SetLineColor(kOrange);            s7x->SetFillColor(kOrange);
          e7x->SetFillStyle(3002);               s7x->SetFillStyle(3001);
          s7x->DrawNormalized();                           e7x->DrawNormalized("same");




  //*********************************************************************//

  TCanvas* cStripY = new TCanvas("cStripY","cStripY",2000,1000);
  cStripY->Divide(4,2);

  cStripY->cd(1);
    tree_both->Draw("Mugast.DSSD_Y>>e1y(128,0,128)","Mugast.TelescopeNumber==1","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s1y(128,0,128)","Mugast.TelescopeNumber==1","same");
    TH1F* e1y = (TH1F*) gDirectory->Get(        "e1y");
    TH1F* s1y = (TH1F*) gDirectory->Get(        "s1y");
          s1y->GetYaxis()->SetRangeUser(0,8000);
          e1y->SetLineColor(kRed);               e1y->SetFillColor(kRed);
          s1y->SetLineColor(kOrange);            s1y->SetFillColor(kOrange);
          e1y->SetFillStyle(3002);               s1y->SetFillStyle(3001);
          s1y->DrawNormalized();                           e1y->DrawNormalized("same");

  cStripY->cd(2);
    tree_both->Draw("Mugast.DSSD_Y>>e2y(128,0,128)","Mugast.TelescopeNumber==2","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s2y(128,0,128)","Mugast.TelescopeNumber==2","same");
    TH1F* e2y = (TH1F*) gDirectory->Get(        "e2y");
    TH1F* s2y = (TH1F*) gDirectory->Get(        "s2y");
          s2y->GetYaxis()->SetRangeUser(0,8000);
          e2y->SetLineColor(kRed);               e2y->SetFillColor(kRed);
          s2y->SetLineColor(kOrange);            s2y->SetFillColor(kOrange);
          e2y->SetFillStyle(3002);               s2y->SetFillStyle(3001);
          s2y->DrawNormalized();                           e2y->DrawNormalized("same");

 cStripY->cd(3);
    tree_both->Draw("Mugast.DSSD_Y>>e3y(128,0,128)","Mugast.TelescopeNumber==3","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s3y(128,0,128)","Mugast.TelescopeNumber==3","same");

    TH1F* e3y = (TH1F*) gDirectory->Get(        "e3y");
    TH1F* s3y = (TH1F*) gDirectory->Get(        "s3y");
          s3y->GetYaxis()->SetRangeUser(0,8000);
          e3y->SetLineColor(kRed);               e3y->SetFillColor(kRed);
          s3y->SetLineColor(kOrange);            s3y->SetFillColor(kOrange);
          e3y->SetFillStyle(3002);               s3y->SetFillStyle(3001);
          s3y->DrawNormalized();                           e3y->DrawNormalized("same");

  cStripY->cd(4);
    tree_both->Draw("Mugast.DSSD_Y>>e4y(128,0,128)","Mugast.TelescopeNumber==4","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s4y(128,0,128)","Mugast.TelescopeNumber==4","same");

    TH1F* e4y = (TH1F*) gDirectory->Get(        "e4y");
    TH1F* s4y = (TH1F*) gDirectory->Get(        "s4y");
          s4y->GetYaxis()->SetRangeUser(0,8000);
          e4y->SetLineColor(kRed);               e4y->SetFillColor(kRed);
          s4y->SetLineColor(kOrange);            s4y->SetFillColor(kOrange);
          e4y->SetFillStyle(3002);               s4y->SetFillStyle(3001);
          s4y->DrawNormalized();                           e4y->DrawNormalized("same");

  cStripY->cd(5);
    tree_both->Draw("Mugast.DSSD_Y>>e5y(128,0,128)","Mugast.TelescopeNumber==5","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s5y(128,0,128)","Mugast.TelescopeNumber==5","same");

    TH1F* e5y = (TH1F*) gDirectory->Get(        "e5y");
    TH1F* s5y = (TH1F*) gDirectory->Get(        "s5y");
          s5y->GetYaxis()->SetRangeUser(0,8000);
          e5y->SetLineColor(kRed);               e5y->SetFillColor(kRed);
          s5y->SetLineColor(kOrange);            s5y->SetFillColor(kOrange);
          e5y->SetFillStyle(3002);               s5y->SetFillStyle(3001);
          s5y->DrawNormalized();                           e5y->DrawNormalized("same");

  cStripY->cd(6);
    tree_both->Draw("Mugast.DSSD_Y>>e6y(128,0,128)","Mugast.TelescopeNumber==6","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s6y(128,0,128)","Mugast.TelescopeNumber==6","same");

    TH1F* e6y = (TH1F*) gDirectory->Get(        "e6y");
    TH1F* s6y = (TH1F*) gDirectory->Get(        "s6y");
          s6y->GetYaxis()->SetRangeUser(0,8000);
          e6y->SetLineColor(kRed);               e6y->SetFillColor(kRed);
          s6y->SetLineColor(kOrange);            s6y->SetFillColor(kOrange);
          e6y->SetFillStyle(3002);               s6y->SetFillStyle(3001);
          s6y->DrawNormalized();                           e6y->DrawNormalized("same");

  cStripY->cd(7);
    tree_both->Draw("Mugast.DSSD_Y>>e7y(128,0,128)","Mugast.TelescopeNumber==7","");
    tree_sim->Draw( "Mugast.DSSD_Y>>s7y(128,0,128)","Mugast.TelescopeNumber==7","same");

    TH1F* e7y = (TH1F*) gDirectory->Get(        "e7y");
    TH1F* s7y = (TH1F*) gDirectory->Get(        "s7y");
          s7y->GetYaxis()->SetRangeUser(0,8000);
          e7y->SetLineColor(kRed);               e7y->SetFillColor(kRed);
          s7y->SetLineColor(kOrange);            s7y->SetFillColor(kOrange);
          e7y->SetFillStyle(3002);               s7y->SetFillStyle(3001);
          s7y->DrawNormalized();                           e7y->DrawNormalized("same");

}

//////////////////////////////////////////////////////////////////////////////////////
void CompareExpSimCountsPerAngle()
{
  TChain* tree_sim=NULL;
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/Sim_19Odp_1675.root");
  tree_sim = Chain("PhysicsTree",files,true);

  //*********************************************************************//

  TCanvas* cCounts = new TCanvas("cCounts","cCounts",1000,1000);

  tree_both->Draw("ThetaLab>>e(120,100,160)","Mugast.TelescopeNumber<8","");
  tree_sim->Draw( "ThetaLab>>s(120,100,160)","Mugast.TelescopeNumber<8","same");
  TH1F* e = (TH1F*) gDirectory->Get("e");
  TH1F* s = (TH1F*) gDirectory->Get("s");
  e->SetLineColor(kBlack);
  s->SetLineColor(kRed); 
  e->DrawNormalized();
  s->DrawNormalized("same");

}

//==================================================================================//

//////////////////////////////////////////////////////////////////////////////////////
void HeavyThetaLabLightThetaLab_ExpPSp()
{
  TChain* tree_psp=NULL;
  files.clear();
  files.push_back("~/Programs/nptoolv3/Outputs/Analysis/PSp_PhaseSpace_0000.root");
  tree_psp = Chain("PhysicsTree",files,true);

  tree_both->Draw("HeavyThetaLab:ThetaLab>>eG(10,104,124,75,0.5,2.0)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2","");
  tree_both->Draw("HeavyThetaLab:ThetaLab>>eB(10,104,124,75,0.5,2.0)","abs(T_FPMW_HF-10300)<1500 && Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2","");
  tree_psp ->Draw("HeavyThetaLab:ThetaLab>>ps(10,104,124,75,0.5,2.0)","EventWeight*(Mugast.TelescopeNumber<9 && ThetaLab>106 && ELab>1.2)","");
  
  TH2F* eG = (TH2F*) gDirectory->Get("eG");
  TH2F* eB = (TH2F*) gDirectory->Get("eB");
  TH2F* ps = (TH2F*) gDirectory->Get("ps");
  
  TH2F* eF = (TH2F*) gDirectory->Get("eG");
    eF->SetTitle("eF");
    eF->SetName("eF");
    eF->Add(eB,-1);
 
  //double ratio=0.00113356 * 2.;
  double ratio = 0.00216125;
  cout << "SCALING PHASE SPACE BY " << ratio << endl;

  eF->ProjectionY("prj105", 0, 1,"");  auto prj105 = (TH1F*) gDirectory->Get("prj105");  prj105->SetLineColor(kBlack);  prj105->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj107", 1, 2,"");  auto prj107 = (TH1F*) gDirectory->Get("prj107");  prj107->SetLineColor(kBlack);  prj107->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj109", 2, 3,"");  auto prj109 = (TH1F*) gDirectory->Get("prj109");  prj109->SetLineColor(kBlack);  prj109->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj111", 3, 4,"");  auto prj111 = (TH1F*) gDirectory->Get("prj111");  prj111->SetLineColor(kBlack);  prj111->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj113", 4, 5,"");  auto prj113 = (TH1F*) gDirectory->Get("prj113");  prj113->SetLineColor(kBlack);  prj113->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj115", 5, 6,"");  auto prj115 = (TH1F*) gDirectory->Get("prj115");  prj115->SetLineColor(kBlack);  prj115->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj117", 6, 7,"");  auto prj117 = (TH1F*) gDirectory->Get("prj117");  prj117->SetLineColor(kBlack);  prj117->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj119", 7, 8,"");  auto prj119 = (TH1F*) gDirectory->Get("prj119");  prj119->SetLineColor(kBlack);  prj119->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj121", 8, 9,"");  auto prj121 = (TH1F*) gDirectory->Get("prj121");  prj121->SetLineColor(kBlack);  prj121->GetYaxis()->SetRangeUser(0,325);
  eF->ProjectionY("prj123", 9,10,"");  auto prj123 = (TH1F*) gDirectory->Get("prj123");  prj123->SetLineColor(kBlack);  prj123->GetYaxis()->SetRangeUser(0,325);

  ps->ProjectionY("psp105", 0, 1,"");  auto psp105 = (TH1F*) gDirectory->Get("psp105");  psp105->SetLineColor(kRed);  psp105->Scale(ratio);
  ps->ProjectionY("psp107", 1, 2,"");  auto psp107 = (TH1F*) gDirectory->Get("psp107");  psp107->SetLineColor(kRed);  psp107->Scale(ratio);
  ps->ProjectionY("psp109", 2, 3,"");  auto psp109 = (TH1F*) gDirectory->Get("psp109");  psp109->SetLineColor(kRed);  psp109->Scale(ratio);
  ps->ProjectionY("psp111", 3, 4,"");  auto psp111 = (TH1F*) gDirectory->Get("psp111");  psp111->SetLineColor(kRed);  psp111->Scale(ratio);
  ps->ProjectionY("psp113", 4, 5,"");  auto psp113 = (TH1F*) gDirectory->Get("psp113");  psp113->SetLineColor(kRed);  psp113->Scale(ratio);
  ps->ProjectionY("psp115", 5, 6,"");  auto psp115 = (TH1F*) gDirectory->Get("psp115");  psp115->SetLineColor(kRed);  psp115->Scale(ratio);
  ps->ProjectionY("psp117", 6, 7,"");  auto psp117 = (TH1F*) gDirectory->Get("psp117");  psp117->SetLineColor(kRed);  psp117->Scale(ratio);
  ps->ProjectionY("psp119", 7, 8,"");  auto psp119 = (TH1F*) gDirectory->Get("psp119");  psp119->SetLineColor(kRed);  psp119->Scale(ratio);
  ps->ProjectionY("psp121", 8, 9,"");  auto psp121 = (TH1F*) gDirectory->Get("psp121");  psp121->SetLineColor(kRed);  psp121->Scale(ratio);
  ps->ProjectionY("psp123", 9,10,"");  auto psp123 = (TH1F*) gDirectory->Get("psp123");  psp123->SetLineColor(kRed);  psp123->Scale(ratio);

  TCanvas* cHTLT = new TCanvas("cHTLT","cHTLT",1500,1500);
  cHTLT->Divide(5,2);
 
  cHTLT->cd( 1); prj105->Draw(); psp105->Draw("same");
  cHTLT->cd( 2); prj107->Draw(); psp107->Draw("same");
  cHTLT->cd( 3); prj109->Draw(); psp109->Draw("same");
  cHTLT->cd( 4); prj111->Draw(); psp111->Draw("same");
  cHTLT->cd( 5); prj113->Draw(); psp113->Draw("same");
  cHTLT->cd( 6); prj115->Draw(); psp115->Draw("same");
  cHTLT->cd( 7); prj117->Draw(); psp117->Draw("same");
  cHTLT->cd( 8); prj119->Draw(); psp119->Draw("same");
  cHTLT->cd( 9); prj121->Draw(); psp121->Draw("same");
  cHTLT->cd(10); prj123->Draw(); psp123->Draw("same");

}

//==================================================================================//

void CompareGammaTrees()
{
  gStyle->SetOptStat(0);
  gStyle->SetPadTickX(1);
  gStyle->SetPadTickY(1);
  gStyle->SetPadGridY(1);
  TCanvas* cCompGamTree = new TCanvas("cCompGamTree","cCompGamTree",1000,1000);
  cCompGamTree->Divide(2,2);

  TLatex text;  text.SetTextFont(132);

  TLine* l1674 = new TLine();  l1674->SetLineColor(kRed);  l1674->SetLineStyle(kDashed);
  TLine* l1897 = new TLine();  l1897->SetLineColor(kRed);  l1897->SetLineStyle(kDashed);
  TLine* l2397 = new TLine();  l2397->SetLineColor(kRed);  l2397->SetLineStyle(kDashed);
  TLine* l3551 = new TLine();  l3551->SetLineColor(kRed);  l3551->SetLineStyle(kDashed);

  //double ymax = 1000;
  double ymax = 180;

  //=================================================================================
  cCompGamTree->cd(2);
  cCompGamTree->cd(2)->SetTopMargin(0.05);
  cCompGamTree->cd(2)->SetRightMargin(0.05);
//tree_au->Draw("trackEDC>>htrack(1000,1,5)","abs(T_FPMW_HF-13300)<1500","");
  tree_cd->Draw("trackEDC>>htrack(1000,1,5)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
  TH1F* htrack = (TH1F*) gDirectory->Get("htrack");
  htrack->GetYaxis()->SetRangeUser(0,ymax);
  htrack->SetTitle("");
  htrack->GetXaxis()->SetTitle("E_{#gamma}");  htrack->GetXaxis()->CenterTitle();
  htrack->GetYaxis()->SetTitle("Counts");  htrack->GetYaxis()->CenterTitle();
  l1674->DrawLine(1.674, 0, 1.674, ymax);
  l1897->DrawLine(1.897, 0, 1.897, ymax);
  l2397->DrawLine(2.397, 0, 2.397, ymax);
  l3551->DrawLine(3.551, 0, 3.551, ymax);
  text.DrawLatex(3.7,0.925*ymax,"trackEDC");
  text.DrawLatex(3.7,0.875*ymax," -> Etrack");
  text.DrawLatex(3.7,0.825*ymax," -> mid-target");
  
  //=================================================================================
  cCompGamTree->cd(1);
  cCompGamTree->cd(1)->SetTopMargin(0.05);
  cCompGamTree->cd(1)->SetRightMargin(0.05);
//tree_au->Draw("gammaEDC>>hgamma(1000,1,5)","abs(T_FPMW_HF-13300)<1500","");
  tree_cd->Draw("gammaEDC>>hgamma(1000,1,5)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
  TH1F* hgamma = (TH1F*) gDirectory->Get("hgamma");
  hgamma->GetYaxis()->SetRangeUser(0,ymax);
  hgamma->SetTitle("");
  hgamma->GetXaxis()->SetTitle("E_{#gamma}");  hgamma->GetXaxis()->CenterTitle();
  hgamma->GetYaxis()->SetTitle("Counts");  hgamma->GetYaxis()->CenterTitle();
  l1674->DrawLine(1.674, 0, 1.674, ymax);
  l1897->DrawLine(1.897, 0, 1.897, ymax);
  l2397->DrawLine(2.397, 0, 2.397, ymax);
  l3551->DrawLine(3.551, 0, 3.551, ymax);
  text.DrawLatex(3.7,0.925*ymax,"gammaEDC");
  text.DrawLatex(3.7,0.875*ymax," -> Egamma");
  text.DrawLatex(3.7,0.825*ymax," -> mid-target");
  
  //=================================================================================
  cCompGamTree->cd(4);
  cCompGamTree->cd(4)->SetTopMargin(0.05);
  cCompGamTree->cd(4)->SetRightMargin(0.05);
//tree_au->Draw("goldEDC >> hgold(1000,1,5)","abs(T_FPMW_HF-13300)<1500","");
  tree_cd->Draw("goldEDC >> hgold(1000,1,5)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
  TH1F* hgold = (TH1F*) gDirectory->Get("hgold");
  hgold->GetYaxis()->SetRangeUser(0,ymax);
  hgold->SetTitle("");
  hgold->GetXaxis()->SetTitle("E_{#gamma}");  hgold->GetXaxis()->CenterTitle();
  hgold->GetYaxis()->SetTitle("Counts");  hgold->GetYaxis()->CenterTitle();
  l1674->DrawLine(1.674, 0, 1.674, ymax);
  l1897->DrawLine(1.897, 0, 1.897, ymax);
  l2397->DrawLine(2.397, 0, 2.397, ymax);
  l3551->DrawLine(3.551, 0, 3.551, ymax);
  text.DrawLatex(3.7,0.925*ymax,"goldEDC");
  text.DrawLatex(3.7,0.875*ymax," -> Etrack");
  text.DrawLatex(3.7,0.825*ymax," -> post-Au");
  
  //=================================================================================
  cCompGamTree->cd(3);
  cCompGamTree->cd(3)->SetTopMargin(0.05);
  cCompGamTree->cd(3)->SetRightMargin(0.05);
//tree_au->Draw("vamosEDC>>hvamos(1000,1,5)","abs(T_FPMW_HF-13300)<1500","");
  tree_cd->Draw("vamosEDC>>hvamos(1000,1,5)","abs(T_FPMW_HF-13300)<1500 && Mugast.TelescopeNumber<9","");
  TH1F* hvamos = (TH1F*) gDirectory->Get("hvamos");
  hvamos->GetYaxis()->SetRangeUser(0,ymax);
  hvamos->SetTitle("");
  hvamos->GetXaxis()->SetTitle("E_{#gamma}");  hvamos->GetXaxis()->CenterTitle();
  hvamos->GetYaxis()->SetTitle("Counts");  hvamos->GetYaxis()->CenterTitle();
  l1674->DrawLine(1.674, 0, 1.674, ymax);
  l1897->DrawLine(1.897, 0, 1.897, ymax);
  l2397->DrawLine(2.397, 0, 2.397, ymax);
  l3551->DrawLine(3.551, 0, 3.551, ymax);
  text.DrawLatex(3.7,0.925*ymax,"vamosEDC");
  text.DrawLatex(3.7,0.875*ymax," -> Egamma");
  text.DrawLatex(3.7,0.825*ymax," -> post-Au");


}

//==================================================================================//

void TWOFNR_ApproachingUnbound()
{
  TCanvas* cTWOFNR = new TCanvas("cTWOFNR","cTWOFNR",1000,1000);
  cTWOFNR->Divide(2,2);
  cTWOFNR->cd(1)->SetLogy();
  cTWOFNR->cd(2)->SetLogy();
  cTWOFNR->cd(3)->SetLogy();
  cTWOFNR->cd(4)->SetLogy();
  
  cTWOFNR->cd(1);
  TGraph* s12_7500 = TWOFNR(7.500,2,1,0,0.5,2.5); s12_7500->SetLineColor(colourR->GetNumber()); s12_7500->SetLineWidth(4); s12_7500->SetLineStyle(1); s12_7500->GetXaxis()->SetRangeUser(100,160); s12_7500->GetYaxis()->SetRangeUser(0.004,2); s12_7500->Draw();
  TGraph* s12_7520 = TWOFNR(7.520,2,1,0,0.5,2.5); s12_7520->SetLineColor(colourB->GetNumber()); s12_7520->SetLineWidth(2); s12_7520->SetLineStyle(7); s12_7520->GetXaxis()->SetRangeUser(100,160); s12_7520->GetYaxis()->SetRangeUser(0.004,2); s12_7520->Draw("same");
  TGraph* s12_7540 = TWOFNR(7.540,2,1,0,0.5,2.5); s12_7540->SetLineColor(colourB->GetNumber()); s12_7540->SetLineWidth(2); s12_7540->SetLineStyle(7); s12_7540->GetXaxis()->SetRangeUser(100,160); s12_7540->GetYaxis()->SetRangeUser(0.004,2); s12_7540->Draw("same");
  TGraph* s12_7560 = TWOFNR(7.560,2,1,0,0.5,2.5); s12_7560->SetLineColor(colourB->GetNumber()); s12_7560->SetLineWidth(2); s12_7560->SetLineStyle(7); s12_7560->GetXaxis()->SetRangeUser(100,160); s12_7560->GetYaxis()->SetRangeUser(0.004,2); s12_7560->Draw("same");
  TGraph* s12_7580 = TWOFNR(7.580,2,1,0,0.5,2.5); s12_7580->SetLineColor(colourB->GetNumber()); s12_7580->SetLineWidth(2); s12_7580->SetLineStyle(7); s12_7580->GetXaxis()->SetRangeUser(100,160); s12_7580->GetYaxis()->SetRangeUser(0.004,2); s12_7580->Draw("same");
  TGraph* s12_7600 = TWOFNR(7.600,2,1,0,0.5,2.5); s12_7600->SetLineColor(colourG->GetNumber()); s12_7600->SetLineWidth(4); s12_7600->SetLineStyle(1); s12_7600->GetXaxis()->SetRangeUser(100,160); s12_7600->GetYaxis()->SetRangeUser(0.004,2); s12_7600->Draw("same");

  cTWOFNR->cd(2);
  TGraph* p12_7500 = TWOFNR(7.500,2,0,1,0.5,2.5); p12_7500->SetLineColor(colourR->GetNumber()); p12_7500->SetLineWidth(4); p12_7500->SetLineStyle(1); p12_7500->GetXaxis()->SetRangeUser(100,160); p12_7500->GetYaxis()->SetRangeUser(0.004,2); p12_7500->Draw();
  TGraph* p12_7520 = TWOFNR(7.520,2,0,1,0.5,2.5); p12_7520->SetLineColor(colourB->GetNumber()); p12_7520->SetLineWidth(2); p12_7520->SetLineStyle(7); p12_7520->GetXaxis()->SetRangeUser(100,160); p12_7520->GetYaxis()->SetRangeUser(0.004,2); p12_7520->Draw("same");
  TGraph* p12_7540 = TWOFNR(7.540,2,0,1,0.5,2.5); p12_7540->SetLineColor(colourB->GetNumber()); p12_7540->SetLineWidth(2); p12_7540->SetLineStyle(7); p12_7540->GetXaxis()->SetRangeUser(100,160); p12_7540->GetYaxis()->SetRangeUser(0.004,2); p12_7540->Draw("same");
  TGraph* p12_7560 = TWOFNR(7.560,2,0,1,0.5,2.5); p12_7560->SetLineColor(colourB->GetNumber()); p12_7560->SetLineWidth(2); p12_7560->SetLineStyle(7); p12_7560->GetXaxis()->SetRangeUser(100,160); p12_7560->GetYaxis()->SetRangeUser(0.004,2); p12_7560->Draw("same");
  TGraph* p12_7580 = TWOFNR(7.580,2,0,1,0.5,2.5); p12_7580->SetLineColor(colourB->GetNumber()); p12_7580->SetLineWidth(2); p12_7580->SetLineStyle(7); p12_7580->GetXaxis()->SetRangeUser(100,160); p12_7580->GetYaxis()->SetRangeUser(0.004,2); p12_7580->Draw("same");
  TGraph* p12_7600 = TWOFNR(7.600,2,0,1,0.5,2.5); p12_7600->SetLineColor(colourG->GetNumber()); p12_7600->SetLineWidth(4); p12_7600->SetLineStyle(1); p12_7600->GetXaxis()->SetRangeUser(100,160); p12_7600->GetYaxis()->SetRangeUser(0.004,2); p12_7600->Draw("same");

  cTWOFNR->cd(3);
  TGraph* d52_7500 = TWOFNR(7.500,2,0,2,2.5,2.5); d52_7500->SetLineColor(colourR->GetNumber()); d52_7500->SetLineWidth(4); d52_7500->SetLineStyle(1); d52_7500->GetXaxis()->SetRangeUser(100,160); d52_7500->GetYaxis()->SetRangeUser(0.004,2); d52_7500->Draw();
  TGraph* d52_7520 = TWOFNR(7.520,2,0,2,2.5,2.5); d52_7520->SetLineColor(colourB->GetNumber()); d52_7520->SetLineWidth(2); d52_7520->SetLineStyle(7); d52_7520->GetXaxis()->SetRangeUser(100,160); d52_7520->GetYaxis()->SetRangeUser(0.004,2); d52_7520->Draw("same");
  TGraph* d52_7540 = TWOFNR(7.540,2,0,2,2.5,2.5); d52_7540->SetLineColor(colourB->GetNumber()); d52_7540->SetLineWidth(2); d52_7540->SetLineStyle(7); d52_7540->GetXaxis()->SetRangeUser(100,160); d52_7540->GetYaxis()->SetRangeUser(0.004,2); d52_7540->Draw("same");
  TGraph* d52_7560 = TWOFNR(7.560,2,0,2,2.5,2.5); d52_7560->SetLineColor(colourB->GetNumber()); d52_7560->SetLineWidth(2); d52_7560->SetLineStyle(7); d52_7560->GetXaxis()->SetRangeUser(100,160); d52_7560->GetYaxis()->SetRangeUser(0.004,2); d52_7560->Draw("same");
  TGraph* d52_7580 = TWOFNR(7.580,2,0,2,2.5,2.5); d52_7580->SetLineColor(colourB->GetNumber()); d52_7580->SetLineWidth(2); d52_7580->SetLineStyle(7); d52_7580->GetXaxis()->SetRangeUser(100,160); d52_7580->GetYaxis()->SetRangeUser(0.004,2); d52_7580->Draw("same");
  TGraph* d52_7600 = TWOFNR(7.600,2,0,2,2.5,2.5); d52_7600->SetLineColor(colourG->GetNumber()); d52_7600->SetLineWidth(4); d52_7600->SetLineStyle(1); d52_7600->GetXaxis()->SetRangeUser(100,160); d52_7600->GetYaxis()->SetRangeUser(0.004,2); d52_7600->Draw("same");

  cTWOFNR->cd(4);
  TGraph* f72_7500 = TWOFNR(7.500,3,0,3,3.5,2.5); f72_7500->SetLineColor(colourR->GetNumber()); f72_7500->SetLineWidth(4); f72_7500->SetLineStyle(1); f72_7500->GetXaxis()->SetRangeUser(100,160); f72_7500->GetYaxis()->SetRangeUser(0.004,2); f72_7500->Draw();
  TGraph* f72_7520 = TWOFNR(7.520,3,0,3,3.5,2.5); f72_7520->SetLineColor(colourB->GetNumber()); f72_7520->SetLineWidth(2); f72_7520->SetLineStyle(7); f72_7520->GetXaxis()->SetRangeUser(100,160); f72_7520->GetYaxis()->SetRangeUser(0.004,2); f72_7520->Draw("same");
  TGraph* f72_7540 = TWOFNR(7.540,3,0,3,3.5,2.5); f72_7540->SetLineColor(colourB->GetNumber()); f72_7540->SetLineWidth(2); f72_7540->SetLineStyle(7); f72_7540->GetXaxis()->SetRangeUser(100,160); f72_7540->GetYaxis()->SetRangeUser(0.004,2); f72_7540->Draw("same");
  TGraph* f72_7560 = TWOFNR(7.560,3,0,3,3.5,2.5); f72_7560->SetLineColor(colourB->GetNumber()); f72_7560->SetLineWidth(2); f72_7560->SetLineStyle(7); f72_7560->GetXaxis()->SetRangeUser(100,160); f72_7560->GetYaxis()->SetRangeUser(0.004,2); f72_7560->Draw("same");
  TGraph* f72_7580 = TWOFNR(7.580,3,0,3,3.5,2.5); f72_7580->SetLineColor(colourB->GetNumber()); f72_7580->SetLineWidth(2); f72_7580->SetLineStyle(7); f72_7580->GetXaxis()->SetRangeUser(100,160); f72_7580->GetYaxis()->SetRangeUser(0.004,2); f72_7580->Draw("same");
  TGraph* f72_7600 = TWOFNR(7.600,3,0,3,3.5,2.5); f72_7600->SetLineColor(colourG->GetNumber()); f72_7600->SetLineWidth(4); f72_7600->SetLineStyle(1); f72_7600->GetXaxis()->SetRangeUser(100,160); f72_7600->GetYaxis()->SetRangeUser(0.004,2); f72_7600->Draw("same");

}

void TWOFNR_1MeVJumps()
{
  TCanvas* cTWOFNR = new TCanvas("cTWOFNR","cTWOFNR",1000,1000);
  cTWOFNR->Divide(2,2);
  cTWOFNR->cd(1)->SetLogy();
  cTWOFNR->cd(2)->SetLogy();
  cTWOFNR->cd(3)->SetLogy();
  cTWOFNR->cd(4)->SetLogy();
  
  cTWOFNR->cd(1);
  TGraph* s12_7500 = TWOFNR(2.000,2,1,0,0.5,2.5); s12_7500->SetLineColor(colourR->GetNumber()); s12_7500->SetLineWidth(4); s12_7500->SetLineStyle(1); s12_7500->GetXaxis()->SetRangeUser(100,160); s12_7500->GetYaxis()->SetRangeUser(0.004,2); s12_7500->Draw();
  TGraph* s12_7520 = TWOFNR(3.000,2,1,0,0.5,2.5); s12_7520->SetLineColor(colourB->GetNumber()); s12_7520->SetLineWidth(2); s12_7520->SetLineStyle(7); s12_7520->GetXaxis()->SetRangeUser(100,160); s12_7520->GetYaxis()->SetRangeUser(0.004,2); s12_7520->Draw("same");
  TGraph* s12_7540 = TWOFNR(4.000,2,1,0,0.5,2.5); s12_7540->SetLineColor(colourB->GetNumber()); s12_7540->SetLineWidth(2); s12_7540->SetLineStyle(7); s12_7540->GetXaxis()->SetRangeUser(100,160); s12_7540->GetYaxis()->SetRangeUser(0.004,2); s12_7540->Draw("same");
  TGraph* s12_7560 = TWOFNR(5.000,2,1,0,0.5,2.5); s12_7560->SetLineColor(colourB->GetNumber()); s12_7560->SetLineWidth(2); s12_7560->SetLineStyle(7); s12_7560->GetXaxis()->SetRangeUser(100,160); s12_7560->GetYaxis()->SetRangeUser(0.004,2); s12_7560->Draw("same");
  TGraph* s12_7580 = TWOFNR(6.000,2,1,0,0.5,2.5); s12_7580->SetLineColor(colourB->GetNumber()); s12_7580->SetLineWidth(2); s12_7580->SetLineStyle(7); s12_7580->GetXaxis()->SetRangeUser(100,160); s12_7580->GetYaxis()->SetRangeUser(0.004,2); s12_7580->Draw("same");
  TGraph* s12_7600 = TWOFNR(7.000,2,1,0,0.5,2.5); s12_7600->SetLineColor(colourG->GetNumber()); s12_7600->SetLineWidth(4); s12_7600->SetLineStyle(1); s12_7600->GetXaxis()->SetRangeUser(100,160); s12_7600->GetYaxis()->SetRangeUser(0.004,2); s12_7600->Draw("same");

  cTWOFNR->cd(2);
  TGraph* p12_7500 = TWOFNR(2.000,2,0,1,0.5,2.5); p12_7500->SetLineColor(colourR->GetNumber()); p12_7500->SetLineWidth(4); p12_7500->SetLineStyle(1); p12_7500->GetXaxis()->SetRangeUser(100,160); p12_7500->GetYaxis()->SetRangeUser(0.004,2); p12_7500->Draw();
  TGraph* p12_7520 = TWOFNR(3.000,2,0,1,0.5,2.5); p12_7520->SetLineColor(colourB->GetNumber()); p12_7520->SetLineWidth(2); p12_7520->SetLineStyle(7); p12_7520->GetXaxis()->SetRangeUser(100,160); p12_7520->GetYaxis()->SetRangeUser(0.004,2); p12_7520->Draw("same");
  TGraph* p12_7540 = TWOFNR(4.000,2,0,1,0.5,2.5); p12_7540->SetLineColor(colourB->GetNumber()); p12_7540->SetLineWidth(2); p12_7540->SetLineStyle(7); p12_7540->GetXaxis()->SetRangeUser(100,160); p12_7540->GetYaxis()->SetRangeUser(0.004,2); p12_7540->Draw("same");
  TGraph* p12_7560 = TWOFNR(5.000,2,0,1,0.5,2.5); p12_7560->SetLineColor(colourB->GetNumber()); p12_7560->SetLineWidth(2); p12_7560->SetLineStyle(7); p12_7560->GetXaxis()->SetRangeUser(100,160); p12_7560->GetYaxis()->SetRangeUser(0.004,2); p12_7560->Draw("same");
  TGraph* p12_7580 = TWOFNR(6.000,2,0,1,0.5,2.5); p12_7580->SetLineColor(colourB->GetNumber()); p12_7580->SetLineWidth(2); p12_7580->SetLineStyle(7); p12_7580->GetXaxis()->SetRangeUser(100,160); p12_7580->GetYaxis()->SetRangeUser(0.004,2); p12_7580->Draw("same");
  TGraph* p12_7600 = TWOFNR(7.000,2,0,1,0.5,2.5); p12_7600->SetLineColor(colourG->GetNumber()); p12_7600->SetLineWidth(4); p12_7600->SetLineStyle(1); p12_7600->GetXaxis()->SetRangeUser(100,160); p12_7600->GetYaxis()->SetRangeUser(0.004,2); p12_7600->Draw("same");

  cTWOFNR->cd(3);
  TGraph* d52_7500 = TWOFNR(2.000,2,0,2,2.5,2.5); d52_7500->SetLineColor(colourR->GetNumber()); d52_7500->SetLineWidth(4); d52_7500->SetLineStyle(1); d52_7500->GetXaxis()->SetRangeUser(100,160); d52_7500->GetYaxis()->SetRangeUser(0.004,2); d52_7500->Draw();
  TGraph* d52_7520 = TWOFNR(3.000,2,0,2,2.5,2.5); d52_7520->SetLineColor(colourB->GetNumber()); d52_7520->SetLineWidth(2); d52_7520->SetLineStyle(7); d52_7520->GetXaxis()->SetRangeUser(100,160); d52_7520->GetYaxis()->SetRangeUser(0.004,2); d52_7520->Draw("same");
  TGraph* d52_7540 = TWOFNR(4.000,2,0,2,2.5,2.5); d52_7540->SetLineColor(colourB->GetNumber()); d52_7540->SetLineWidth(2); d52_7540->SetLineStyle(7); d52_7540->GetXaxis()->SetRangeUser(100,160); d52_7540->GetYaxis()->SetRangeUser(0.004,2); d52_7540->Draw("same");
  TGraph* d52_7560 = TWOFNR(5.000,2,0,2,2.5,2.5); d52_7560->SetLineColor(colourB->GetNumber()); d52_7560->SetLineWidth(2); d52_7560->SetLineStyle(7); d52_7560->GetXaxis()->SetRangeUser(100,160); d52_7560->GetYaxis()->SetRangeUser(0.004,2); d52_7560->Draw("same");
  TGraph* d52_7580 = TWOFNR(6.000,2,0,2,2.5,2.5); d52_7580->SetLineColor(colourB->GetNumber()); d52_7580->SetLineWidth(2); d52_7580->SetLineStyle(7); d52_7580->GetXaxis()->SetRangeUser(100,160); d52_7580->GetYaxis()->SetRangeUser(0.004,2); d52_7580->Draw("same");
  TGraph* d52_7600 = TWOFNR(7.000,2,0,2,2.5,2.5); d52_7600->SetLineColor(colourG->GetNumber()); d52_7600->SetLineWidth(4); d52_7600->SetLineStyle(1); d52_7600->GetXaxis()->SetRangeUser(100,160); d52_7600->GetYaxis()->SetRangeUser(0.004,2); d52_7600->Draw("same");

  cTWOFNR->cd(4);
  TGraph* f72_7500 = TWOFNR(2.000,3,0,3,3.5,2.5); f72_7500->SetLineColor(colourR->GetNumber()); f72_7500->SetLineWidth(4); f72_7500->SetLineStyle(1); f72_7500->GetXaxis()->SetRangeUser(100,160); f72_7500->GetYaxis()->SetRangeUser(0.004,2); f72_7500->Draw();
  TGraph* f72_7520 = TWOFNR(3.000,3,0,3,3.5,2.5); f72_7520->SetLineColor(colourB->GetNumber()); f72_7520->SetLineWidth(2); f72_7520->SetLineStyle(7); f72_7520->GetXaxis()->SetRangeUser(100,160); f72_7520->GetYaxis()->SetRangeUser(0.004,2); f72_7520->Draw("same");
  TGraph* f72_7540 = TWOFNR(4.000,3,0,3,3.5,2.5); f72_7540->SetLineColor(colourB->GetNumber()); f72_7540->SetLineWidth(2); f72_7540->SetLineStyle(7); f72_7540->GetXaxis()->SetRangeUser(100,160); f72_7540->GetYaxis()->SetRangeUser(0.004,2); f72_7540->Draw("same");
  TGraph* f72_7560 = TWOFNR(5.000,3,0,3,3.5,2.5); f72_7560->SetLineColor(colourB->GetNumber()); f72_7560->SetLineWidth(2); f72_7560->SetLineStyle(7); f72_7560->GetXaxis()->SetRangeUser(100,160); f72_7560->GetYaxis()->SetRangeUser(0.004,2); f72_7560->Draw("same");
  TGraph* f72_7580 = TWOFNR(6.000,3,0,3,3.5,2.5); f72_7580->SetLineColor(colourB->GetNumber()); f72_7580->SetLineWidth(2); f72_7580->SetLineStyle(7); f72_7580->GetXaxis()->SetRangeUser(100,160); f72_7580->GetYaxis()->SetRangeUser(0.004,2); f72_7580->Draw("same");
  TGraph* f72_7600 = TWOFNR(7.000,3,0,3,3.5,2.5); f72_7600->SetLineColor(colourG->GetNumber()); f72_7600->SetLineWidth(4); f72_7600->SetLineStyle(1); f72_7600->GetXaxis()->SetRangeUser(100,160); f72_7600->GetYaxis()->SetRangeUser(0.004,2); f72_7600->Draw("same");


cout << "FROM ENERGY 2 MeV TO 7 MeV" << endl;
}

//==================================================================================//

double AGATAefficiency(double gammaE_MeV)
{
  double A = 1.0;
  double B = 1.0;
  double C = 0.000; //Fixed to 0, hence not included in IZanon's thesis equation
  double D = 1.0;   //IZ: C
  double E = 1.0;   //IZ: D
  double F = 1.0;   //IZ: E
  double G = 1.0;   //IZ: F
  
  double constX = log(gammaE_MeV / 0.1); //x = log(Eg/100 keV)
  double constY = log(gammaE_MeV / 1.0); //x = log(Eg/  1 MeV)

  double exponent1 = pow(A + (B * constX) + (C * constX * constX) , -G);
  double exponent2 = pow(D + (E * constY) + (F * constY * constY) , -G);

  double exponentF = pow(exponent1+exponent2, -1/G);

  double effic = exp(exponentF);


  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;
  cout << " DO NOT HAVE THE VARIABLED FO THE FUNCTIONS! INVALID! " << endl;


  return effic;
}
